"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ApiDecoratorRegistry_1 = require("../ApiDecoratorRegistry");
/**
 * decorators for controller endpoint methods, a path
 * for the endpoint can be specified
 */
function GET(path = "") {
    return (classDefinition, methodName) => registerApiEndpoint(classDefinition, methodName, path, "GET");
}
exports.GET = GET;
function POST(path = "") {
    return (classDefinition, methodName) => registerApiEndpoint(classDefinition, methodName, path, "POST");
}
exports.POST = POST;
function PUT(path = "") {
    return (classDefinition, methodName) => registerApiEndpoint(classDefinition, methodName, path, "PUT");
}
exports.PUT = PUT;
function DELETE(path = "") {
    return (classDefinition, methodName) => registerApiEndpoint(classDefinition, methodName, path, "DELETE");
}
exports.DELETE = DELETE;
function PATCH(path = "") {
    return (classDefinition, methodName) => registerApiEndpoint(classDefinition, methodName, path, "PATCH");
}
exports.PATCH = PATCH;
function registerApiEndpoint(classDefinition, methodName, path, httpMethod) {
    ApiDecoratorRegistry_1.ApiDecoratorRegistry.Endpoints.push((api, controllerFactory) => {
        // build a instance of the associated controller
        let controller = controllerFactory(classDefinition.constructor);
        let call = mapHttpMethodToCall(api, httpMethod);
        let endpointPath = path;
        let controllerDefintion = classDefinition.constructor;
        if (controllerDefintion.rootPath) {
            // prepend controller root path to endpoint path 
            endpointPath = `${controllerDefintion.rootPath}${path}`;
        }
        // call api setup method, passing in handler that will invoke 
        // the endpoint method on the controller instance with request 
        // and response objects
        call(endpointPath, (req, res) => controller[methodName](req, res));
    });
}
function mapHttpMethodToCall(api, method) {
    if (method == "GET") {
        return api.get;
    }
    else if (method == "POST") {
        return api.post;
    }
    else if (method == "PUT") {
        return api.put;
    }
    else if (method == "PATCH") {
        return api.patch;
    }
    else if (method == "DELETE") {
        return api.delete;
    }
    throw "Unrecognised HTTP method '" + method + "'";
}
//# sourceMappingURL=Endpoints.js.map